import decimal
from django.db import models
from django.test import TestCase

class Bug38Table(models.Model):
    d = models.DecimalField(max_digits=5, decimal_places=2)


class Bug38TestCase(TestCase):
    def testInsertVariousFormats(self):
        """
        Test adding decimals as strings with various formats.
        """
        Bug38Table(d=decimal.Decimal('0')).save()
        Bug38Table(d=decimal.Decimal('0e0')).save()
        Bug38Table(d=decimal.Decimal('0E0')).save()
        Bug38Table(d=decimal.Decimal('450')).save()
        Bug38Table(d=decimal.Decimal('450.0')).save()
        Bug38Table(d=decimal.Decimal('450.00')).save()
        Bug38Table(d=decimal.Decimal('450.000')).save()
        Bug38Table(d=decimal.Decimal('0450')).save()
        Bug38Table(d=decimal.Decimal('0450.0')).save()
        Bug38Table(d=decimal.Decimal('0450.00')).save()
        Bug38Table(d=decimal.Decimal('0450.000')).save()
        Bug38Table(d=decimal.Decimal('4.5e+2')).save()
        Bug38Table(d=decimal.Decimal('4.5E+2')).save()
        self.assertEquals(len(list(Bug38Table.objects.all())),13)

    def testReturnsDecimal(self):
        """
        Test if return value is a python Decimal object 
        when saving the model with a Decimal object as value 
        """
        Bug38Table(d=decimal.Decimal('0')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal, d1.d.__class__)

    def testReturnsDecimalFromString(self):
        """
        Test if return value is a python Decimal object 
        when saving the model with a unicode object as value.
        """
        Bug38Table(d=u'123').save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal, d1.d.__class__)        

    def testSavesAfterDecimal(self):
        """
        Test if value is saved correctly when there are numbers 
        to the right side of the decimal point 
        """
        Bug38Table(d=decimal.Decimal('450.1')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal('450.1'), d1.d)
    
    def testInsertWithMoreDecimals(self):
        """
        Test if numbers to the right side of the decimal point 
        are saved correctly rounding to a decimal with the correct 
        decimal places.
        """
        Bug38Table(d=decimal.Decimal('450.111')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal('450.11'), d1.d)    
        
    def testInsertWithLeadingZero(self):
        """
        Test if value is saved correctly with Decimals with a leading zero.
        """
        Bug38Table(d=decimal.Decimal('0450.0')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal('450.0'), d1.d)


        



from django.db import models, connection, backend, load_backend

from django.core import signals
from django.db import close_connection
from django.core.handlers.base import BaseHandler

from django.db.backends.oracle.base import DatabaseWrapper as OracleDatabaseWrapper
from oraclepool.base import DatabaseWrapper as PoolDatabaseWrapper
from django.core.paginator import Paginator
from django.test import TestCase
from datetime import datetime
from settings import DATABASES, get_settings_dict
from random import randint

class OneTable(models.Model):
    b = models.CharField(max_length=100)
    c = models.CharField(default=u'test', max_length=10)
    
    def __repr__(self):
        return '<OneTable %s: %s, %s>' % (self.pk, self.b, self.c)


class TwoTable(models.Model):
    a = models.ForeignKey(OneTable)
    b = models.CharField(max_length=100)
    
    def __repr__(self):
        return '<TwoTable %s: %s>' % (self.pk, self.b)

class PerformanceTestCase(TestCase):
    """ Runs a set of inserts and queries via each oracle
        connector and times the overall execution to indicate
        pool connection creation time differences
     """

    conncount = 64 # Apache2 default is 2 processes with 64 threads = 128
    querycount = 60  # multiply by five for number of queries
    conns = {'oracle':[],
             'oraclepool':[]}
    rows = {'oracle':0,
            'oraclepool':0}
    newconns = 1
    start = datetime.now()
    time = {}
    
    def resetTimer(self):
        self.start = datetime.now()        
        self.time = {'oracle':0,
                     'oraclepool':0}
        
    
    def testTimed(self):
        """ Main test sequence is run here other funcs are helpers
            for those timed tests
        """
        fixtures = ['paging.json']
        self.setup_conns()
        conncount = {}
        for i in range(0,2):
            self.newconns = i
            self.resetTimer()
            print "\n-----------------------------------------------"            
            if i == 0:
                print 'Run query speed test set using a single connection'
                DATABASES['oraclepool']['EXTRAS']['min'] = 1
                DATABASES['oraclepool']['EXTRAS']['increment'] = 0
            else:
                print 'Run test set using real world multiple process thread held connections'
                DATABASES['oraclepool']['EXTRAS']['min'] = 4
                DATABASES['oraclepool']['EXTRAS']['increment'] = 2
                DATABASES['oraclepool']['EXTRAS']['max'] = 8                
            print "-----------------------------------------------\n"
            
            for engine in ['oracle','oraclepool']:
                DATABASES['default'] = DATABASES[engine]
                self.addData(engine)
                self.queryData(engine)
                OneTable.objects.all().delete()
                conncount[engine] = 1 - i
                for conn in self.conns[engine]:
                    if conn:
                        conncount[engine] +=1
                print '%s engine took %s microsecs with %s connections' % (engine,
                                                               self.time[engine],
                                                               conncount[engine])
            self.assertEquals(self.rows['oracle'],self.rows['oraclepool'])
            print 'For each engine ran %d queries' % int(10 + (self.querycount * 5))
            speed = int(100*self.time['oracle']/self.time['oraclepool'])
            if speed > 100:
                adj = 'faster than'
            else:
                adj = 'of the speed of'
            print 'Oraclepool is %d percent %s Oracle engine' % (speed,adj)

    def addData(self, engine):
        self.dummy_request_start(engine)
        OneTable(b='here').save()
        OneTable(b='is').save()
        OneTable(b='some').save()
        OneTable(b='test').save()
        OneTable(b='data').save()
        OneTable(b='for').save()
        OneTable(b='querying').save()
        OneTable(b='fgh').save()
        OneTable(b='fgh').save()
        OneTable(b='ijk').save()
        self.dummy_request_end(engine)        

    def queryData(self, engine):
        """ Do a lot of queries """
        for i in range(0, self.querycount):
            self.dummy_request_start(engine)
            self.runQuery(engine, b__startswith='f')
            self.runQuery(engine, b__endswith='h')
            self.runQuery(engine, b__contains='e')        
            self.runQuery(engine, twotable__b__startswith='B')
            self.runQuery(engine, twotable__b__exact='B1')        
            self.dummy_request_end(engine)

    def runQuery(self, engine, **kwargs):
        """ Switch engine each time """
        Q = OneTable.objects.filter(**kwargs)
        for rec in Q:
            retrieve = rec
            self.rows[engine] += 1
        return

    def setup_conns(self):
        """ The global connection object is rebuilt frequently
            just as it is in deployment with a multi- process and/or threaded
            web server - such as apache2.
            This unpooled multiple connection use is where the bulk of
            the time is lost when the connections dont use pooling.
            
            TODO: Probably should do proper child process forking for holding
            each of these to more realistically simulate a production web server
        """
        for engine in ['oraclepool','oracle']:        
            for i in range(0, self.conncount):
                self.conns[engine].append(None)

    def get_connection(self, engine):
        """ The global connection object is switched whenever 
            a different process or threads serves the request
            NB: Must call cursor for it to actually connect to oracle
            and hence demonstrate the real world pooled connection effect
        """
        if self.newconns:
            globals()['connection'].close()
            globals()['connection'] = None
            DATABASES['default'] = DATABASES[engine]
            backend = load_backend(DATABASES[engine]['ENGINE'])
            process = randint(0,self.conncount - 1)
            if not self.conns[engine][process]:
                self.conns[engine][process] = backend.DatabaseWrapper(get_settings_dict(engine)) 
                cursor = self.conns[engine][process].cursor()
            globals()['connection'] = self.conns[engine][process]
        return

    def dummy_request_start(self, engine):
        """ Send the normal signals and start the timer """
        self.start = datetime.now()
        self.get_connection(engine)
        globals()['connection'].queries = []        
        signals.request_started.send(sender=self.__class__)

    def dummy_request_end(self, engine):
        """ Send the normal signals and end the timer """        
        signals.request_finished.disconnect(close_connection)
        signals.request_finished.send(sender=self.__class__)
        signals.request_finished.connect(close_connection)
        delta = datetime.now() - self.start
        self.time[engine] += delta.microseconds



import settings
from django.db.backends.oracle.creation import DatabaseCreation
from datetime import datetime

# Monkeypatch standard oracle driver for tests 
# so it can be tested against an existing db too. See settings.
if settings.DATABASES['oracle']['EXTRAS'].get('existing',''):

    print 'Monkey patching for using existing database for tests'

    def _create_test_db(self, verbosity=1, autoclobber=False):
        """ if existing is set then this uses the settings database
            for testing rather than creating a new one
        """
        self.start_time(True)
        if not getattr(settings, 'TEST_DATABASE_NAME', ''):
            settings.TEST_DATABASE_NAME = settings.DATABASE_NAME
            settings.TEST_DATABASE_USER = settings.DATABASE_USER
            settings.TEST_DATABASE_PASSWD = settings.DATABASE_PASSWORD
        print 'Using Test Database %s' % settings.TEST_DATABASE_NAME
        return settings.TEST_DATABASE_NAME

    def start_time(self, reset=False):
        """ Getter / setter since attrib may be lost with monkey patch """
        if reset or not getattr(self, 'start', None):
            self.start = datetime.now()
        return self.start

    def _destroy_test_db(self, test_database_name, verbosity=1):
        """ if existing is set then this must clean up all the test
            schema and data - not just drop the database
        """
        print "#### Built tables and tested in %s ####" % str(datetime.now() - self.start_time())
        from django.db import connection
        print 'Cleaning up test data and schema from %s' %  settings.TEST_DATABASE_NAME
        from oraclepool.creation import DatabaseCreation as DCPool
        pool = DCPool(connection)
        pool._drop_test_tables()
        pool._delete_test_users()

    DatabaseCreation.start_time = start_time
    DatabaseCreation._create_test_db = _create_test_db
    DatabaseCreation._destroy_test_db = _destroy_test_db

    





from paging.models import *

a1 = FirstTable(b='A1')
a1.save()

a2 = FirstTable(b='A2')
a2.save()

b1 = SecondTable(a=a1, b='B1')
b1.save()

b2 = SecondTable(a=a1, b='B2')
b2.save()

b3 = SecondTable(a=a1, b='B3')
b3.save()



import datetime
import decimal
from django.db import models
from django.test import TestCase


class BaseModel(models.Model):
    def __unicode__(self):
        return u'id: ' + unicode(self.id) + ' Amount: ' + unicode(self.val)

    class Meta:
        abstract = True

class TableNullTime(BaseModel):
    """
    This test isn't expected to work on SQL Server 2005,
    as there is no bare "time" type.
    
    TableNullTime(val=None).save()
    TableNullTime(val=datetime.time(2,34,2)).save()
    len(list(TableNullTime.objects.all()))
    2
    """
    val = models.TimeField(null=True)


class TableNullChar(BaseModel):
    val = models.CharField(null=True, max_length=100)
    
class TableNullText(BaseModel):
    val = models.TextField(null=True)

class TableNullInteger(BaseModel):
    val = models.IntegerField(null=True)

class TableNullDateTime(BaseModel):
    val = models.DateTimeField(null=True)

class TableNullDate(BaseModel):
    val = models.DateField(null=True)

class TableNullNullBoolean(BaseModel):
    val = models.NullBooleanField(null=True)

class TableNullDecimal(BaseModel):
    val = models.DecimalField(null=True, max_digits=4, decimal_places=2)

class TableNullFloat(BaseModel):
    val = models.FloatField(null=True)


class NullTests(TestCase):
    def _run(self, model, values):
        for v in values:
            model(val=v).save()
            
        self.assertEquals(len(list(model.objects.all())), len(values))

    def testChar(self):
        self._run(TableNullChar, (None, "This is my string value."))
        
    def testText(self):
        self._run(TableNullText, (None, "This is my string value."))
        
    def testInteger(self):
        self._run(TableNullInteger, (None, 32768))
        
    def testDateTime(self):
        self._run(TableNullDateTime,(None, datetime.datetime(2009,1,1,4,3,5)))
     
    def testDate(self):
        self._run(TableNullDate, (None, datetime.date(2009,1,1)))
     
    def testBoolean(self):
        self._run(TableNullNullBoolean, (None, True, False))

    def testFloat(self):
        self._run(TableNullFloat, (None, 34.3))

    def testDecimal(self):
        self._run(TableNullDecimal, (None, decimal.Decimal('99.99')))



from django.db import models
from django.db.models import (
    Avg, Count, Max, Min, StdDev, Sum, Variance)

class AmountTable(models.Model):
    """Test that basic aggregates work.

    >>> AmountTable(amount=100).save()
    >>> AmountTable(amount=101).save()
    >>> AmountTable(amount=102).save()
    >>> len(list(AmountTable.objects.all()))
    3

    >>> AmountTable.objects.aggregate(Avg('amount'))
    {'amount__avg': 101.0}

    >>> AmountTable.objects.aggregate(Max('amount'))
    {'amount__max': 102}

    >>> AmountTable.objects.aggregate(Min('amount'))
    {'amount__min': 100}

    >>> AmountTable.objects.aggregate(Sum('amount'))
    {'amount__sum': 303}
    """
    amount = models.IntegerField()

class Player(models.Model):
    name = models.CharField(max_length=40)
    
class GamerCard(models.Model):
    player = models.ForeignKey(Player)
    name = models.CharField(max_length=40, default="default_card")
    avatar = models.CharField(max_length=40)
    
class Bet(models.Model):
    """Test cross-table aggregates
    
    >>> p1 = Player(name="Adam Vandenberg")
    >>> p1.save()
    
    >>> p2 = Player(name="Joe Betsalot")
    >>> p2.save()
    
    >>> GamerCard(player=p1).save()
    >>> GamerCard(player=p2).save()
    
    >>> Bet(player=p1, amount="100.00").save()
    >>> Bet(player=p1, amount="200.00").save()
    >>> Bet(player=p1, amount="300.00").save()
    >>> Bet(player=p1, amount="400.00").save()
    >>> Bet(player=p1, amount="500.00").save()
    
    >>> Bet(player=p2, amount="1000.00").save()
    >>> Bet(player=p2, amount="2000.00").save()
    >>> Bet(player=p2, amount="3000.00").save()
    >>> Bet(player=p2, amount="4000.00").save()
    >>> Bet(player=p2, amount="5000.00").save()
    
    >>> p = Player.objects.annotate(Count('bet'), avg_bet=Avg('bet__amount')).order_by('name')
    
    >>> p[0].name
    u'Adam Vandenberg'
    >>> p[0].bet__count
    5
    >>> p[0].avg_bet
    300
    
    >>> p[1].name
    u'Joe Betsalot'
    >>> p[1].bet__count
    5
    >>> p[1].avg_bet
    3000
    
    >>> p = Player.objects.annotate(bets=Count('bet'), avg_bet=Avg('bet__amount')).values()
    """
    player = models.ForeignKey(Player)
    amount = models.DecimalField(max_digits=10, decimal_places=2)



#!/usr/bin/env python
import sys, os
# import django and oracle pool based on the location of this tests folder ...
here_path = os.path.abspath(os.path.dirname("."))
os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

# Specify path to django if its not installed in pythons own libs
# and not found above the location of this egg
# sys.path.append('your path to django')
try:
    from django.core.management import execute_manager
except:
    egg_path = os.path.join('django-oraclepool','tests')
    djangopath = here_path.replace(egg_path, '')
    parts = os.path.split(djangopath)
    while parts[0]:
        djangopath = os.path.join(parts[0],'django')
        if os.path.exists(djangopath):
            sys.path.append(djangopath)
            break
        djangopath = parts[0]
        parts = os.path.split(djangopath)
    from django.core.management import execute_manager
    
# Add oracle pool path
oraclepool_path = here_path.replace('tests', '')
sys.path.append(oraclepool_path)
	
try:
    import settings # Assumed to be in the same directory.
except ImportError:
    sys.stderr.write("Error: Can't find the file 'settings.py' in the directory containing %r. It appears you've customized things.\nYou'll have to run django-admin.py, passing it your settings module.\n(If the file settings.py does indeed exist, it's causing an ImportError somehow.)\n" % __file__)
    sys.exit(1)

if __name__ == "__main__":
    execute_manager(settings)



from django.conf.urls.defaults import *

urlpatterns = patterns('',
    # Example:
    # (r'^test_models/', include('test_models.foo.urls')),

    # Uncomment this for admin:
     (r'^admin/', include('django.contrib.admin.urls')),
)



from setuptools import setup, find_packages
import os

version = '0.9'

setup(name='django-oraclepool',
      version=version,
      description="django database backend that uses cx_Oracle session pooling for connections",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "TODO.txt")).read() + "\n" +      
                       open(os.path.join("docs", "HISTORY.txt")).read(),      
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Django",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='django oracle connection pooling cx_Oracle',
      author='Ed Crewe, Taras Halturin',
      author_email='ed.crewe@bris.ac.uk',
      url='http://bitbucket.org/edcrewe/django-oraclepool',
      license='Apache',
      packages=['oraclepool'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
